/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.federation.servlet;

//core java classes
import java.io.IOException;
import java.net.*;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

//core servlet classes
import javax.servlet.ServletException;
import javax.servlet.UnavailableException;
import javax.servlet.http.*;
import javax.servlet.*;

//jmx related
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.jboss.mx.util.MBeanProxy;
import org.jboss.mx.util.MBeanServerLocator;


//logging
import org.apache.log4j.Logger;

import org.jboss.security.federation.service.FederationService;
import org.jboss.security.federation.model.Partner;

//jboss-sso framework classes
import org.jboss.security.saml.SingleSignOn;
import org.jboss.security.saml.SSOManager;
import org.jboss.security.saml.AuthResponse;
import org.jboss.security.sso.util.SSOUtil;

//jboss-security-common classes
import org.jboss.security.util.Tools;

/**
 * The Federation Server. For details please look at <a target="_blank" href="http://labs.jboss.com/portal/index.html?ctrl:cmd=render&ctrl:window=default.wiki.WikiPortletWindow&page=FederationServer&language=EN">Federation Server</a>
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class SSOFederationServer extends HttpServlet
{
    /**
     * 
     */
    private static final Logger log = Logger.getLogger(SSOFederationServer.class);
    private static final String token = "token"; //the SSO token        
    private static final String target = "target"; //the resource on the partner site that needs to be accessed
    private static final String partnerConfig = "partners"; //name of the init parameter that has the value of the partners configuration file
    private static final String logout = "logout";
        
    /**
     * 
     */
    private static Map partners = new HashMap();
    
    /**
     * 
     */
    private FederationService fedService = null;
    
    /**
     * 
     */
    public void init() throws ServletException
    {
        try
        {
            ServletConfig config = this.getServletConfig();        
            SSOFederationServer.partners = new HashMap();
            
            //get the registered federation service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            this.fedService = (FederationService)MBeanProxy.get
            (FederationService.class,
            new ObjectName("jboss.sso:service=FederationServer"), 
            mbeanServer);
            
            Collection partners = this.fedService.getPartners();
            if(partners!=null)
            {
                for(Iterator itr=partners.iterator();itr.hasNext();)
                {
                    Partner partner = (Partner)itr.next();
                    SSOFederationServer.partners.put(partner.getDomain(),partner.getFederationServer());
                }
            }
            
            log.info("Configuration successfully loaded for the SSOFederationServer...........");
        }
        catch(Exception e)
        {
            log.error(this,e);
            SSOFederationServer.partners = new HashMap();
            throw new UnavailableException(e.getMessage());            
        }                        
    }
    
        
    /**
     * 
     */
    public void doPost(HttpServletRequest request,HttpServletResponse response)
    throws ServletException,IOException
    {  
    	if(request.getParameterMap()==null || request.getParameterMap().size()==0)
        {
            //send a view of the partners and return
            request.setAttribute("partners",SSOFederationServer.partners);
            request.getRequestDispatcher("/WEB-INF/viewPartners.jsp").forward(request,response);
            return;
        }
                
        //perform token processing here
        try
        {                        
            String tokenParam = request.getParameter(SSOFederationServer.token);
            //process a token from the partner and forward to the resource in your domain
            if(tokenParam!=null && tokenParam.trim().length()>0)
            {
                //this is an RP request to process (relying party needs to process the cookie)
                //from a party that has already asserted the identity of the user
                String target = request.getParameter(SSOFederationServer.target);
                String token = SSOManager.cleanToken(request.getParameter(SSOFederationServer.token));
                String tokenCookie = SSOManager.cleanToken(this.getToken(request));
                                
                if(                        
                        !token.equals(SSOFederationServer.logout) //make sure this is not a federated logout command
                        //if its not a federated logout command, it is the federated sso token
                )
                {
                    String federationUser = SSOUtil.getUsername(token);
                    String loggedInUser = SSOUtil.getUsername(tokenCookie);
                    
                    if(federationUser==null)
                    {
                        federationUser = "";                        
                    }
                    
                    if(loggedInUser==null)
                    {
                        loggedInUser = "";
                    }
                    
                    if(federationUser.trim().length()>0 && !federationUser.trim().equals(loggedInUser.trim()))
                    {
                        AuthResponse authResponse = SSOUtil.getAuthResponse(token);
                        String myToken = SSOUtil.generateToken(authResponse);
                        
                        //create the same cookie from the domain that the user is coming from into my domain
                        Cookie rpCookie = new Cookie(SSOFederationServer.token,myToken);
                        URL url = new URL(target);
                        String host = url.getHost();
                        
                        if(host!=null && host.trim().length()>0 && 
                           host.indexOf('[')==-1 //this makes sure an IPv6 address with a [xxx.xx] is not sent in as the domain of the cookie
                        )
                        {
                            String domain = this.getDomain(host);
                            if(domain!=null && domain.trim().length()>0)
                            {
                                rpCookie.setDomain(domain);
                            }
                        }
                        rpCookie.setPath("/"); //make this a top-level cookie
                        response.addCookie(rpCookie);
                    }
                }
                else
                {
                    //go ahead and delete the sso cookie for this domain
                    //get the domain for setting a domain cookie
                    String domain = this.getDomain(request.getServerName());
                    
                    //create a domain cookie                    
                    Cookie logoutCookie = new Cookie(SingleSignOn.SSO_TOKEN,"logout");
                    logoutCookie.setDomain(domain);
                    logoutCookie.setPath("/");
                    logoutCookie.setMaxAge(0); //this should delete this cookie
                    
                    response.addCookie(logoutCookie);
                }
                
                
                //forward to the resource in your domain
                request.setAttribute("target", target);
                request.getRequestDispatcher("/WEB-INF/target.jsp").forward(request,response);
            }            
            else
            {
                //this is the asserting party, need to extract the token from this domain and forward to
                //the relying party's federation server, the token will be transported to that domain                        
                String target = request.getParameter(SSOFederationServer.target); //resource in that domain being accessed
                String token = this.getToken(request);
                
                //get the federation server for this partner
                String federationServer = this.getFederationServer(target);
                
                //make this a post redirect rather than a query string get due to the
                //limitations on the size of query string
                if(federationServer!=null && federationServer.trim().length()>0)
                {
                    //create the url to re-direct to
                    String url = federationServer;
                    
                    //setup the url properly
                    if(!url.endsWith("/"))
                    {
                        url += "/";
                    }
                    
                    if(token==null || token.trim().length()==0)
                    {
                        //federated logout
                        token = SSOFederationServer.logout;
                    }
                    
                                                           
                    request.setAttribute("url",url);
                    request.setAttribute("target",target);
                    request.setAttribute("token",Tools.escapeStr(token));
                    request.getRequestDispatcher("/WEB-INF/transportToken.jsp").forward(request,response);
                }
                else
                {
                    //a federation server for this partner was not found....go back to where this request was issued from
                    String referer = request.getHeader("Referer");
                    response.sendRedirect(referer);
                }
            }
        }
        catch(Exception e)
        {
            log.error(this,e);
            response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }    	    	
    }
    
    /**
     * 
     */
    public void doGet(HttpServletRequest request,HttpServletResponse response)
    throws ServletException,IOException
    {
    	String contextPath = request.getContextPath();
        String requestURI = request.getRequestURI();
        String actionPath = requestURI.substring(contextPath.length());
        
        if(actionPath.endsWith(partnerConfig))
        {
        	String xml = this.fedService.getPartnerInfo();
        	response.getOutputStream().write(xml.getBytes());
        	response.getOutputStream().flush();
        }
        else
        {
        	this.doPost(request, response);
        }
    }
    
    
    //utility methods-------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     */
    private String getFederationServer(String target) throws Exception
    {
        String federationServer = null;
        
        URL url = new URL(target);
        String host = url.getHost();
        
        if(host!=null && host.trim().length()>0 && 
           host.indexOf('[')==-1 //this makes sure an IPv6 address with a [xxx.xx] is not sent in as the domain of the cookie
        )
        {
            String domain = this.getDomain(host);
            if(domain!=null && domain.trim().length()>0)
            {
                if(domain.startsWith("."))
                {
                    domain = domain.substring(1);
                }
                
                //get the federation server for this domain
                federationServer = (String)SSOFederationServer.partners.get(domain);
            }
        }                
        return federationServer;
    }
    
    /**
     * 
     * @param request
     * @return
     */
    private String getToken(HttpServletRequest request)
    {
        String token = null;
        Cookie[] cookies = request.getCookies();
        if(cookies!=null)
        {
            for(int i=0;i<cookies.length;i++)
            {
                if(cookies[i].getName().equals(SSOFederationServer.token))
                {
                    token = cookies[i].getValue();
                    break;
                }
            }
        }            
        return token;
    }
    
    /**
     * 
     * @param serverName
     * @return
     */
    private String getDomain(String serverName)
    {
        String domain = null;

        boolean looksLikeIP = serverName.matches("^([0-9]{1,3}\\.){3}[0-9]{1,3}$");

        if (looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            looksLikeIP = parts.length == 4;
            for (int i = 0; i < parts.length && looksLikeIP; i++)
            {
                try
                {
                    int xxx = Integer.parseInt(parts[i]);
                    if ((xxx < 0) || (xxx > 255))
                    {
                        looksLikeIP = false;
                    }
                }
                catch (NumberFormatException e)
                {
                    looksLikeIP = false;
                }
            }
        }

        if (!looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            if (parts.length > 2)
            {
                StringBuffer domainStringBuffer = new StringBuffer(".");
                domainStringBuffer.append(parts[parts.length-2]);
                domainStringBuffer.append(".");
                domainStringBuffer.append(parts[parts.length-1]);
                domain = domainStringBuffer.toString();
            }
        }

        return domain;
    }    
}
