/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;


/**
 * The interface <b>org.jboss.security.idm.ProvisioningProvider</b> represents a data source component that extracts data from Identity Stores like
 * LDAP repositories, ActiveX Directories, thirdparty identity management systems like SiteMinder, or even custom JDBC based systems.<br/><br/>
 * 
 * ProvisioningProvider provides an abstraction for Identity Stores to extract Identity data related to Profile Management of users in the system.<br/><br/>
 * 
 * The next release of SSO system will rely on ProvisioningProvider extensively to perform Federation User Provisioning for data stored in scattered Identity Stores.
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Sep 20, 2006
 *
 */
public interface ProvisioningProvider extends LoginProvider
{
    /**
     * Activates a user account
     * 
     * @param identity data of the user account that should be activated
     * @throws IdentityException
     */    
    public void activate(Identity identity) throws IdentityException;
    
    /**
     * Activates a user account
     * 
     * @param username of the user account that should be activated
     * @throws IdentityException
     */
    public void activate(String username) throws IdentityException;
    
    
    /**
     * Deactivates a user account
     * 
     * @param identity data of the user account that should be deactivated
     * @throws IdentityException
     */
    public void deactivate(Identity identity) throws IdentityException;
    
    /**
     * Deactivates a user account
     * 
     * @param username of the user account that should be deactivated
     * @throws IdentityException
     */
    public void deactivate(String username) throws IdentityException;
    
    /**
     * Updates the user profile data
     * 
     * @param identity
     * @throws IdentityException
     */
    public void update(Identity identity) throws IdentityException;
    
    /**
     * Registers a new user into the system
     * 
     * @param identity
     * @throws IdentityException
     */
    public void register(Identity identity) throws IdentityException;
    
    /**
     * Removes a user from the system
     * 
     * @param identity
     * @throws IdentityException
     */
    public void remove(Identity identity) throws IdentityException;
    
    /**
     * Removes a user from the system
     * 
     * @param username
     * @throws IdentityException
     */
    public void remove(String username) throws IdentityException;
    
    /**
     * Resets the password of the specified user
     * 
     * @param username
     * @param password
     * @throws IdentityException
     */
    public void resetPassword(String username,byte[] password) throws IdentityException;    
    //-----search related interface-------------------------------------------------------------------------
    
    //coming soon..............
        
    //--role membership related interface-------------------------------------------------------------------
    /**
     * Checks if the specified role exists in the system
     * 
     * @param role
     * @return
     * @throws IdentityException
     */
    public boolean exists(Role role) throws IdentityException;
    
    /**
     * Reads role information for the specified role name
     * 
     * @param roleName
     * @return
     * @throws IdentityException
     */
    public Role readRole(String roleName) throws IdentityException;
            
    /**
     * Adds a new role to the system
     * 
     * @param role
     * @throws IdentityException
     */
    public void addRole(Role role) throws IdentityException;
    
    /**
     * Removes a role from the system
     * 
     *  @param role
     *  @throws IdentityException
     */
    public void removeRole(Role role) throws IdentityException;
    
    /**
     * Updates the specified role
     * 
     * @param role
     * @throws IdentityException
     */
    public void updateRole(Role role) throws IdentityException;
}
