/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.sso.util;

import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

import org.jboss.security.saml.AuthResponse;
import org.jboss.security.saml.SSOException;
import org.jboss.security.saml.SSOManager;
import org.jboss.security.saml.SingleSignOn;

import org.jboss.security.util.Tools;

/**
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class SSOUtil 
{
    /**
     * Returns the SSOToken which is one of the cookies in the http request coming in
     * 
     * @param request
     * @param tokenName
     * @return
     */
    public static String getSSOToken(HttpServletRequest request,String tokenName)
    {
        String ssoToken = null;
        
        //find the SSOToken cookie
        Cookie[] cookies = request.getCookies();
        if(cookies!=null)
        {
            for(int i=0;i<cookies.length;i++)
            {
                Cookie cookie = cookies[i];
                
                if(cookie.getName()!=null && cookie.getName().trim().equals(tokenName))
                {
                    ssoToken = SSOUtil.cleanSSOToken(cookie.getValue());
                    break;
                }
            }
        }
        
        return ssoToken;
    }
    
    /**
     * Removes unwanted characters from the token coming in as a browser cookie
     * 
     * @param ssoToken
     * @return
     */
    public static String cleanSSOToken(String ssoToken)
    {
        String cleanToken = null;
        
        if(ssoToken!=null && ssoToken.trim().length()>0)
        {
            cleanToken = ssoToken;
            
            //replace ' \" ' with plain ' " '
            cleanToken = Tools.replaceStr(cleanToken, "\\\"", "\"");
            
            //remove the starting ' " '
            if(cleanToken.startsWith("\""))
            {
                cleanToken = cleanToken.substring(1);
            }
            
            //remove the ending ' " '
            if(cleanToken.endsWith("\""))
            {
                cleanToken = cleanToken.substring(0,cleanToken.length()-1);
            }            
        }
        
        return cleanToken;
    }
    
    /**
     * Utility method that extracts the username from the SSO token
     * 
     * @param ssoToken
     * @return
     * @throws SSOException
     */
    public static String getUsername(String ssoToken) throws SSOException
    {
        String username = null;
        
        if(ssoToken!=null && ssoToken.trim().length()>0)
        {
            SingleSignOn sso = SSOManager.findProcessor();
            AuthResponse authInfo = sso.parseAuthResponse(ssoToken);
            
            if(authInfo.getUser()!=null)
            {
                username = authInfo.getUser().getUserName();
            }
        }
        
        return username;
    }
    
    /**
     * Utility method that produces the AuthResponse object from the SSO Token
     * 
     * @param ssoToken
     * @return
     * @throws SSOException
     */
    public static AuthResponse getAuthResponse(String ssoToken) throws SSOException
    {
        AuthResponse response = null;
        
        if(ssoToken!=null && ssoToken.trim().length()>0)
        {
            SingleSignOn sso = SSOManager.findProcessor();
            response = sso.parseAuthResponse(ssoToken);                        
        }
        
        return response;
    }
    
    /**
     * Utility method which checks the validity of an SSO token
     * 
     * @param ssoToken
     * @return
     * @throws SSOException
     */
    public static boolean isTokenValid(String ssoToken) throws SSOException
    {
        boolean isValid = false;
        
        if(ssoToken!=null && ssoToken.trim().length()>0)
        {
            SingleSignOn sso = SSOManager.findProcessor();
            AuthResponse authInfo = sso.parseAuthResponse(ssoToken);
            
            isValid = authInfo.isAuthenticated();
        }
        
        return isValid;
    }
    
    /**
     * Utility method for generating an SSO token from the input AuthResponse object
     * 
     * @param response
     * @return
     */
    public static String generateToken(AuthResponse response) throws SSOException
    {
        String token = null;
        
        SingleSignOn sso = SSOManager.findProcessor();
        token = sso.generateAuthResponse(response.getAssertingParty(),response.getUser().getUserName(),true);
        
        return token;
    }
    
    /**
     * This method returns a SAML (xml) representation of a map of partners that are part of
     * this Single Sign On federation
     * 
     * @param partners - a map of partners belonging to the SSO Federation
     * @return
     * @throws SSOException
     */
    public static String marshalPartners(Map partners)
    throws SSOException
    {
    	SingleSignOn sso = SSOManager.findProcessor();
    	return sso.marshalPartners(partners);
    }
    
    /**
     * This method unmarshals a SAML (xml) representation of partners that are part of this Single Sign On
     * Federation 
     * It returns a map of these partners with
     * key - domain of the partner
     * value - the address of the federation server of this partner
     * 
     * @return
     * @throws SSOException
     */
    public static Map unmarshalPartners(String partners)
    throws SSOException
    {
    	SingleSignOn sso = SSOManager.findProcessor();
    	return sso.unmarshalPartners(partners);
    }
}
