/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.valve;


//core java classes
import java.io.IOException;
import java.security.Principal;


//servlet classes
import javax.servlet.ServletException;
import javax.servlet.http.*;


//core catalina classes for tomcat specific functions
import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.valves.ValveBase;

//JBoss-SSO Framework classes
//jboss-sso-saml library
import org.jboss.security.saml.SingleSignOn;
import org.jboss.security.saml.SSOManager;
import org.jboss.security.saml.SSOException;
import org.jboss.security.sso.util.SSOUtil;

//jboss-sso-common library
import org.jboss.security.util.Tools;

/**
 * The SSOTokenManager Tomcat Valve intercepts requests and checks the presence of an SSOToken domain cookie
 * If the user is logged in (Principal is set) and an SSOToken domain cookie does not exist, it creates one for this user
 * for the duration of this session. It should also remove the domain cookie if the user is logged out (No prinicipal found)
 * 
 * <br/><br/>
 * This valve is to be used for non-JAAS based authentication mechanisms
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 */
public class PlainSSOTokenManager extends ValveBase
{
    /**
     * properties
     */
    private String assertingParty = "";
    
    /**
     * 
     * @return
     */
    public String getAssertingParty()
    {
        return this.assertingParty;
    }
    
    /**
     * 
     * @param assertingParty
     */
    public void setAssertingParty(String assertingParty)
    {
        this.assertingParty = assertingParty;
    }
    
    /**
     * 
     */
    public void invoke(Request request, Response response)
    throws IOException, ServletException 
    {
        try
        {               
            boolean ssoCookieFound = false;  
            boolean isLogoutInProgress = false;
            String ssoToken = null;
            
            //find the SSOToken cookie
            ssoToken = SSOUtil.getSSOToken(request,SingleSignOn.SSO_TOKEN);
            ssoCookieFound = Tools.isCookieFound(request,SingleSignOn.SSO_TOKEN);
            isLogoutInProgress = Tools.isCookieFound(request,SingleSignOn.LOGOUT_TOKEN);
            
            
            
            SSOSession ssoSession = SSOSession.getSSOSession(request.getSession());                                                           
            //if the user is already logged in, make sure the ssoCookie is not processed for automatic login
            if(     ssoCookieFound && 
                    (ssoSession.getPrincipal()!=null //this prevents redundant logins 
                            || 
                     isLogoutInProgress //the prevents autologins during the logout usecase...without this check, the user is never able to logout
                     )
            )
            {
                //stop any automatic sso logins due to dependent requests(images,css etc) that send in the SSO token
                ssoSession.setTurnOff(new Boolean(true));
            }
            
            //clear out the block on automatic sso login
            if(!ssoCookieFound)
            {
                //turns automatic login using SSO back on
                ssoSession.setTurnOff(null);
            }
            
                        
            this.getNext().invoke(request, response);
            
                                                
            //perform any token management functions now....
            Principal principal = null;
            if(request.getSession(false)!=null)
            {
                SSOSession cour = SSOSession.getSSOSession(request.getSession(false));
                principal = cour.getPrincipal();
            }
            if(principal!=null && principal.getName()!=null)
            {                                    
                if(!ssoCookieFound)
                {
                    String domain = this.getDomain(request.getServerName());  
                    //try to set an SSO domain level cookie if a domain of the server being used can be extracted
                    if(domain!=null && domain.trim().length()>0)
                    {
                        //if the SSO domain cookie is not found generate an SSOToken for this user, and set it as the domain cookie
                        SingleSignOn sso = SSOManager.findProcessor();
                        String authInfo = sso.generateAuthResponse(this.assertingParty,principal.getName(),true);
                                            
                                                                
                        //create a domain cookie                    
                        Cookie ssoCookie = new Cookie(SingleSignOn.SSO_TOKEN,authInfo);
                        ssoCookie.setDomain(domain);
                        ssoCookie.setPath("/");
                        
                        //set the domain cookie                         
                        response.addCookie(ssoCookie);
                        ssoSession.setMonitor(new Boolean(true));
                    }
                }
            }  
            else
            {
                //there is no user principal, either user never logged in or logged out,
                //if proper conditions are met, this cookie should be destroyed
                if(ssoCookieFound)
                {                    
                    String requestURI = request.getRequestURI();                    
                    String contextPath = ssoSession.getContext();
                    String requestContext = request.getContextPath();
                    boolean cookieRemoved = false;
                    if(contextPath!=null && contextPath.equals(requestContext))
                    {
                        String ssoTokenPrincipal = SSOUtil.getUsername(ssoToken);
                        String myPrincipal = ssoSession.getUsername();                                                
                        if(ssoTokenPrincipal==null)
                        {
                            ssoTokenPrincipal = "";
                        }                        
                        if(myPrincipal==null)
                        {
                            myPrincipal = "";
                        }
                        
                        //delete the SSOToken only if the SSOSession being destroyed is in sync with the actual SSOToken cookie
                        //if the SSOSession currently active is not for the principal being represented within the SSOToken cookie
                        //don't delete that SSOToken cookie....this can put the federated SSO state between the sites out-of-sync
                        if(ssoTokenPrincipal.trim().length()>0 && ssoTokenPrincipal.trim().equals(myPrincipal.trim()))
                        {
                            this.removeSSOCookie(request,response);
                            cookieRemoved = true;
                        }                                                                        
                    }
                    
                    //if a logout needs to be forced due to an invalid/expired sso cookie
                    if(!cookieRemoved && ssoSession.getForceLogout())
                    {
                        this.removeSSOCookie(request,response);
                    }
                }
            }
        }
        catch(SSOException ssoe)
        {
            throw new ServletException(ssoe);
        }
    }
    
        
    /**
     * 
     * @param serverName
     * @return
     */
    private String getDomain(String serverName)
    {
        String domain = null;

        boolean looksLikeIP = serverName.matches("^([0-9]{1,3}\\.){3}[0-9]{1,3}$");

        if (looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            looksLikeIP = parts.length == 4;
            for (int i = 0; i < parts.length && looksLikeIP; i++)
            {
                try
                {
                    int xxx = Integer.parseInt(parts[i]);
                    if ((xxx < 0) || (xxx > 255))
                    {
                        looksLikeIP = false;
                    }
                }
                catch (NumberFormatException e)
                {
                    looksLikeIP = false;
                }
            }
        }

        if (!looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            if (parts.length > 2)
            {
                StringBuffer domainStringBuffer = new StringBuffer(".");
                domainStringBuffer.append(parts[parts.length-2]);
                domainStringBuffer.append(".");
                domainStringBuffer.append(parts[parts.length-1]);
                domain = domainStringBuffer.toString();
            }
        }

        return domain;
    }
    
    /**
     * 
     * @param response
     */
    private void removeSSOCookie(Request request,Response response)
    {
        //get the domain for setting a domain cookie
        String domain = this.getDomain(request.getServerName());                        
        //create a domain cookie                    
        Cookie ssoCookie = new Cookie(SingleSignOn.SSO_TOKEN,"logout");
        ssoCookie.setDomain(domain);
        ssoCookie.setPath("/");
        ssoCookie.setMaxAge(0); //this should delete this cookie
        response.addCookie(ssoCookie); //delete the ssoToken
        
        //create a logout cookie, this cookies signifies logout is in progress
        //this helps against blocking automatic sso logins when logout is in progress
        //provided the ssoToken is in sync with the ssoSession being destroyed
        Cookie logoutCookie = new Cookie(SingleSignOn.LOGOUT_TOKEN,"logout in progress....");
        logoutCookie.setPath("/");
        logoutCookie.setMaxAge(2); //disable any autologins for two seconds                                           
        response.addCookie(logoutCookie); //signal that a logout is in progress
    }
}
