/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.valve;


//core java classes
import java.io.IOException;
import java.security.Principal;


//servlet classes
import javax.servlet.ServletException;


//core jboss specific classes


//core catalina classes for tomcat specific functions
import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.deploy.LoginConfig;
import org.apache.catalina.valves.ValveBase;
import org.apache.catalina.Context;
import org.apache.catalina.Session;
import org.apache.catalina.authenticator.Constants;


//JBoss-SSO Framework classes
//jboss-sso-saml library
import org.jboss.security.saml.SingleSignOn;
import org.jboss.security.saml.SSOException;
import org.jboss.security.sso.util.SSOUtil;

//jboss-identity-management classes
import org.jboss.security.idm.Identity;
import org.jboss.security.idm.IdentityManager;
import org.jboss.security.idm.LoginProvider;




/**
 * The SSOAutoLogin Tomcat Form Authenticator intercepts requests and checks the presence of an SSOToken domain cookie.
 * If a domain cookie is found it is processed and a Principal is generated resulting in AutoLogin
 * 
 * <br/><br/>
 * This valve is to be used for JAAS based authentication mechanism
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 * 
 */
public class SSOAutoLogin extends ValveBase
{
        
    //----------------------------------------------------- Instance Variables
    /**
     * custom valve configuration info
     */
    private String authType = null;
    private String provider = null;

    /**
     * Descriptive information about this implementation.
     */
    private static final String info = "org.jboss.security.valve.SSOAutoLogin/1.0";


    // ------------------------------------------------------------- Properties


    /**
     * Return descriptive information about this Valve implementation.
     */
    public String getInfo() 
    {
        return (info);
    }
    
    //-----------custom valve configuration----------------------------------------
    /**
     * 
     */
    public String getAuthType() 
    {
        return authType;
    }

    /**
     * 
     * @param authType
     */
    public void setAuthType(String authType) 
    {
        this.authType = authType;
    }
    
    /**
     * 
     * @return
     */
    public String getProvider() 
    {
        return provider;
    }

    /**
     * 
     * @param provider
     */
    public void setProvider(String provider) 
    {
        this.provider = provider;
    }

    /**
     * 
     */
    public void invoke(Request request, Response response)
    throws IOException, ServletException 
    {
        try
        {
            boolean wasSSOPerformed = false;
            boolean performSSO = false;
            SSOSession activeSession = SSOSession.getSSOSession(request.getSession(true));
            
            //check for the block on automatic sso login, if one is found, don't perform the automatic login
            //without this, the user is never able to logout, because all dependent requests to "logout usecase",
            //fire up the automatic sso login
            if(activeSession.getTurnOff()==null)
            {
                performSSO = true;
            }
            
                    
            if(performSSO)
            {
                wasSSOPerformed = this.performSSO(request,response);
            }
            
            //this is only if SSO automated login happened on this request
            this.getNext().invoke(request,response);
            
            //post-valve processing
            if(!wasSSOPerformed)
            {
                //if I get here then no SSO processing was done. perform the regular form based authentication
                if( 
                        request.getSessionInternal(false)!=null && 
                        request.getSessionInternal(false).getPrincipal()!=null && 
                        SSOSession.getSSOSession(request.getSession()).getPrincipal()==null
                 )
                {                    
                    SSOSession.getSSOSession(request.getSession()).
                    setPrincipal(
                            request.getContextPath(),
                            request.getSessionInternal(false).getPrincipal());
                }
            }                        
        }
        catch(SSOException ssoe)
        {
            ssoe.printStackTrace();
            throw new ServletException(ssoe);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new ServletException(e);
        }
    } 
    
    
    /**
     * 
     * @param request
     * @param response
     * @return
     */
    private boolean performSSO(Request request,Response response) throws IOException,SSOException,Exception
    {
        boolean wasSSOPerformed = false;        
        String ssoToken = null;
        boolean ssoCookieFound = false;
        
        //find the SSOToken cookie and setup the proper state
        ssoToken = SSOUtil.getSSOToken(request,SingleSignOn.SSO_TOKEN);
        if(ssoToken!=null && ssoToken.trim().length()>0)
        {
            if(SSOUtil.isTokenValid(ssoToken))
            {
                ssoCookieFound = true;
            }
            else
            {
                //this is an invalid ssotoken, trigger an autologout
                SSOSession ssoSession = SSOSession.getSSOSession(request.getSession());
                ssoSession.setForceLogout(true);
            }
        }
                        
        if(ssoCookieFound)
        {
            Principal principal = request.getSessionInternal(true).getPrincipal();
            if(principal==null)
            {
                //perform auto login for this principal
                LoginConfig config = ((Context)this.container).getLoginConfig();
                String username = SSOUtil.getUsername(ssoToken);
                String password = this.getPassword(username);
                request.setAttribute(SingleSignOn.SSO_USERNAME,username);
                request.setAttribute(SingleSignOn.SSO_PASSWORD,password);
                
                
                boolean ssoLogin = this.ssoLogin(request,response,config);
                if(ssoLogin)
                {                    
                    String contextPath = request.getContextPath();
                    SSOSession ssoSession = SSOSession.getSSOSession(request.getSession());
                    ssoSession.setPrincipal(contextPath,
                    request.getSessionInternal(true).getPrincipal());                                    
                    wasSSOPerformed = true;
                }
            }                                    
        }        
        return wasSSOPerformed;
    }
 
    /**
     * 
     * @param request
     * @param response
     * @param config
     * @return
     * @throws IOException
     */
    private boolean ssoLogin(Request request,Response response,LoginConfig config)
    throws IOException 
    {
        boolean success = false;
        
        Principal principal = null;
        
        // Validate any credentials already included with this request
        String username = (String)request.getAttribute(SingleSignOn.SSO_USERNAME);
        String password = (String)request.getAttribute(SingleSignOn.SSO_PASSWORD);        
        principal = ((Context)this.container).getRealm().authenticate(username, password);
        if (principal != null) 
        {
            this.register(request, response, principal,this.authType,username, password);
            SSOSession.getSSOSession(request.getSession()).setMonitor(new Boolean(true));
            success = true;
        }                
        return success;
    }
    
    /**
     * Register an authenticated Principal and authentication type in our
     * request, in the current session (if there is one), and with our
     * SingleSignOn valve, if there is one.  Set the appropriate cookie
     * to be returned.
     *
     * @param request The servlet request we are processing
     * @param response The servlet response we are generating
     * @param principal The authenticated Principal to be registered
     * @param authType The authentication type to be registered
     * @param username Username used to authenticate (if any)
     * @param password Password used to authenticate (if any)
     */
    private void register(Request request, Response response,
                            Principal principal, String authType,
                            String username, String password) 
    {        
        // Cache the authentication information in our request
        request.setAuthType(authType);
        request.setUserPrincipal(principal);

        Session session = request.getSessionInternal(false);
        // Cache the authentication information in our session, if any
        if (session != null) 
        {
            session.setAuthType(authType);
            session.setPrincipal(principal);
            if (username != null)
                session.setNote(Constants.SESS_USERNAME_NOTE, username);
            else
                session.removeNote(Constants.SESS_USERNAME_NOTE);
            if (password != null)
                session.setNote(Constants.SESS_PASSWORD_NOTE, password);
            else
                session.removeNote(Constants.SESS_PASSWORD_NOTE);
        }
    }
        
    
    /**
     * 
     */
    private String getPassword(String username) throws Exception
    {
        String password = null;
        
        //perform check against JBoss' central identity data (stored in LDAP)
        LoginProvider loginProvider = null;
        if(this.provider==null || this.provider.trim().length()==0)
        {
            loginProvider = IdentityManager.findLoginProvider();
        }
        else
        {
            loginProvider = IdentityManager.findLoginProvider(provider);
        }
        
        if(
                loginProvider.exists(username)
         )
        {
            Identity identity = loginProvider.read(username);                                                                                    
            password = new String(identity.getPassword());
        }
        
        return password;
    }    
}
